;****************************************************************************
;****************************************************************************
;
; isreflec.asm - support routines for use with the debugging aid ISR.EXE
;
;	Copyright (c) 1991
;
;	Rich Bono (NM1D)
;	7 Redfield Circle
;	Derry, New Hampshire, 03038
;
;	All Rights Reserved
;
;	Revisons:
;
;	1.01 01-Feb-91  Tried to make this model independant to allow ease of
;			 use with all models except TINY.  To build the TINY
;			 model, /dMODEL_TINY must be specifed on the TASM
;			 command line when this module is built.  Moved the
;			 local copy of Banner into the CODE segment to make
;			 this fairly model independant.
;
;	1.00 21-Jan-91	First version
;
;
;	Notes:
;
;   1) This is ONLY for use with the provided ISR.EXE resident service.
;
;   2) This MUST be rebuilt if to be used in a TINY model application, see
;      the source code for information.
;
;   3) Assemble with the /ml switch (to preserve case)
;      To rebuild the module you can use the supplied MAKEFILE by
;      invoking the command:
;                MAKE ISREFLEC.OBJ
;
;	         to manually build invoke:
;		    TASM /ml ISREFLEC.ASM
;			(or to build the tiny model)
;		    TASM /ml /dMODEL_TINY ISREFLEC.ASM
;
; TOOLS used:
;
;    Borland TASM 2.0
;
;****************************************************************************
;****************************************************************************
;
;	ISReflect() will install the provided address of your Interrupt
;                   Service Routine to the ISR.EXE resident service.  If
;                   ISR.EXE has not been loaded, this function will do
;                   NOTHING.  This allows for debugging a program and not
;                   having to worry about removing this debug function.
;                   Later for your production code, you can remove this
;                   function (use conditional assembly directives).
;
;	'C' calling convention (for all but TINY models)
;
;		/* preferred method (must be used if ISREFLEC.H is included) */
;		void far ISReflect( (void far *) ptr );
;
;		       or
;
;		/* alternate method */
;		void far ISReflect(unsigned int offset, unsigned int segment);
;
;	'C++' calling convention (for all but TINY models)
;
;		extern "C" void far ISReflect( (void far *) ptr);
;
;
;	To turn off the reflection call ISReflect, provide an address of 0:0
;
;		i.e.: ISReflect( (void far *) 0L);
;
;	[for TINY model: void near ISReflect( (void far *) ptr ); ]
;
;	'ASM' calling convention:
;
;		IF @Model EQ 1	;If TINY model
;		   extrn ISReflect:near
;		ELSE		;Else SMALL, COMPACT, MEDIUM, LARGE, HUGE
;		   extrn ISReflect:far
;		ENDIF
;
;		mov	ax, seg YourISR    ;Push segment of YourISR routine
;		push	ax	
;		mov	ax, offset YourISR ;Push offset of YourISR routine
;		push	ax
;		call	_ISReflect         ;Install the reflection
;		add	sp, 4              ; remove parameters from stack
;
;	or to turn OFF reflection:
;
;		mov	ax, 0              ;Push address of 0:0
;		push	ax
;		push	ax
;		call	_ISReflect         ;Turn reflection off
;		add	sp, 4              ; remove parameters from stack
;
;****************************************************************************
;****************************************************************************
;
; BUGBUG!!! Note: During development of this I found a documentation error
;                 in the Borland Turbo Assembler (2.0) Reference Guide.
;
;      According to the manual, @Model should have the following
;      values depending on the use of .MODEL directive:
;
;	TINY	= 0
;	SMALL	= 1
;	COMPACT	= 2
;	MEDIUM	= 3
;	LARGE	= 4
;	HUGE	= 5
;
;	What I actually found (with TASM 2.0 assembler) was the following:
;
;	TINY	= 1
;	SMALL	= 2
;	COMPACT	= 3
;	MEDIUM	= 4
;	LARGE	= 5
;	HUGE	= 6
;
;****************************************************************************

include isr.inc

;
; If you want to build a TINY model, 
; then include  /dMODEL_TINY on the TASM command line
; 

IFDEF MODEL_TINY	;If /dMODEL_TINY was specified on the command line

  .MODEL TINY		;Build tiny model

ELSE

  .MODEL SMALL	;Should not need to be changed for SMALL, COMPACT, MEDIUM
  		; LARGE or HUGE models.  This has not been extensivly tested.
  		;If you have trouble, change this to your target memory model.
  		;
		;Warning: If significant changes are made to this
		; module, then this may change!

ENDIF

	;
	; args structure describes data on the stack 
	;  (arguments to the function)
	;

args	struc
      IF @Model EQ 1			  ;if TINY model
	retip		dw	1 dup (?) ; Near return address
      ELSE				  ;else SMALL, MEDIUM, LARGE, HUGE
	retip		dd	1 dup (?) ; Far return address
      ENDIF
	savbp		dw	1 dup (?)
	ArgOffset	dw	1 dup (?)	;Offset	 (parameter 1)
	ArgSegment	dw	1 dup (?)	;Segment (parameter 2)
args	ends

;----------------------------------------------------------------------------
.CODE
	;
	; Yes, this is in the code segment!
	;
	;  This was moved here so as to make this module farily model
	;  independant.
	;

Banner	label byte
	BANNERSTRING		;Text of the banner (defined in ISR.INC)
BANNERLEN equ $ - Banner	; length of the Banner string
	db	EOF

;
;----------------------------------------------------------------------------
;
	public _ISReflect

IF @Model EQ 1
 _ISReflect proc near		;If TINY model, this this MUST be near
ELSE
 _ISReflect proc far		;Force this to a FAR procedure for all memory
				; models except TINY.
ENDIF
	push	bp
	mov	bp, sp

	push	es		;Save all used registers so caller doesn't
	push	ds		; have to think about what registers this
	push	ax		; might destroy.
	push	bx
	push	cx
	push	dx
	push	si
	push	di
	pushf

	; Get address of ISR
	mov	ah, GIV			;DOS Get Interrupt Vector service
	mov	al, CMDINT		; ISR vector to retrieve
	int	DOSINT			; returns ES:BX with interrupt vector

	;
	; compare: DS:SI points to the local copy of the Banner
	;          ES:DI is pointing at installed Banner
	; 
	mov	si, cs			;Get code segment into DS (local copy
	mov	ds, si			; of the banner is kept in the code
	ASSUME	ds:@code		; segment)
	
	
	mov	si, offset Banner	; Get offset of local Banner

	mov	di, 0			; Banner offset in ISR
	mov	cx, BANNERLEN		; length of Banner
	cld				; set compare direction
	repe cmpsb			; Compare code

	jne	NotInstalled		;ISR is NOT installed, so don't attempt
					; to call the ISR service!

	;
	; Ok, ISR seems to be installed, so give it our vector address
	;

	mov	dx, [bp].ArgSegment	;Desired segment -> DS
	mov	ds, dx			;
	ASSUME ds:nothing
	mov	dx, [bp].ArgOffset	;Desired offset -> DX
	mov	ax, S_FLAG		;Special ISR flag
	int	CMDINT			; Give this data to ISR resident code

NotInstalled:
	popf
	pop	di			;Restore used registers
	pop	si
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	pop	ds
	pop	es

	pop	bp
	ret

_ISReflect endp
;----------------------------------------------------------------------------

	end

; end of isreflec.asm
